@@ -32,6 +32,7 @@
 
 #ifdef HAVE_DIRSRV_SLAPI_PLUGIN_H
 #include <nspr.h>
+#include <plhash.h>
 #include <nss.h>
 #include <dirsrv/slapi-plugin.h>
 #else
@@ -53,6 +54,9 @@
 #include "map.h"
 #include "back-sch.h"
 
+static void
+backend_entries_to_return_push(struct backend_search_cbdata *cbdata, Slapi_Entry *e);
+
 #define SCH_CONTAINER_CONFIGURATION_FILTER "(&(" SCH_CONTAINER_CONFIGURATION_GROUP_ATTR "=*)(" SCH_CONTAINER_CONFIGURATION_BASE_ATTR "=*)(" SCH_CONTAINER_CONFIGURATION_FILTER_ATTR "=*)(" SCH_CONTAINER_CONFIGURATION_RDN_ATTR "=*))"
 
 /* Read the name of the NIS master. A dummy function for the schema
@@ -415,6 +419,115 @@ backend_set_operational_attributes(Slapi_Entry *e,
 	}
 }
 
+#ifdef USE_NSSWITCH
+#define IPA_ATTR_EXTERNAL_MEMBER "ipaExternalMember"
+#define IPA_ATTR_MEMBERUID "memberUid"
+static void
+backend_set_process_external_members(Slapi_PBlock *pb,
+				   Slapi_Entry *e,
+				   struct plugin_state *state,
+				   struct backend_set_data *data)
+{
+	Slapi_Attr *attr = NULL;
+	Slapi_ValueSet *valueset = NULL;
+	bool_t is_attr_exists, is_group_exists;
+	struct backend_staged_search staged = {0, };
+	struct backend_search_cbdata cbdata = {0, };
+	char *plugin_id = state->plugin_desc->spd_id;
+
+	is_attr_exists = slapi_entry_attr_find(e, IPA_ATTR_EXTERNAL_MEMBER, &attr) == 0;
+
+	if (!is_attr_exists || attr == NULL) {
+		return;
+	}
+
+	/* There are external members in this entry, do group lookup via SSSD
+	 * and update entry's memberUid attribute */
+
+	staged.name = slapi_entry_attr_get_charptr(e, "cn");
+	staged.type = SCH_NSSWITCH_GROUP;
+	staged.search_members = FALSE;
+	staged.is_id = FALSE;
+	staged.is_sid = FALSE;
+	staged.container_sdn = (char*) slapi_sdn_get_dn(data->container_sdn);
+	staged.entries = NULL;
+	staged.count = 0;
+	cbdata.nsswitch_buffer_len = MAX(16384, MAX(sysconf(_SC_GETPW_R_SIZE_MAX), sysconf(_SC_GETGR_R_SIZE_MAX)));
+	cbdata.nsswitch_buffer = malloc(cbdata.nsswitch_buffer_len);
+	cbdata.state = state;
+	cbdata.staged = &staged;
+
+	slapi_log_error(SLAPI_LOG_PLUGIN, plugin_id,
+			"refreshing group membership for group \"%s\"\n", staged.name);
+
+	do {
+		/* This group must exist because it exists in the original tree
+		 * but as dirsrv was restarted, SSSD might still consider its domain offline. */
+		is_group_exists = backend_retrieve_from_nsswitch(&staged, &cbdata);
+		if (!is_group_exists) {
+			slapi_log_error(SLAPI_LOG_FATAL, plugin_id,
+					"group \"%s\" does not exist because SSSD is offline.\n",
+					staged.name);
+			if (state->ready_to_serve == 0) {
+				/* Only wait for SSSD when we populate the original set */
+				slapi_log_error(SLAPI_LOG_FATAL, plugin_id,
+						"waiting for SSSD to become online...\n");
+				DS_Sleep(PR_SecondsToInterval(35));
+			} else {
+				break;
+			}
+		}
+	} while (!is_group_exists);
+
+	if (staged.entries != NULL && staged.entries[0] != NULL) {
+			attr = NULL;
+			if (slapi_entry_attr_find(staged.entries[0], IPA_ATTR_MEMBERUID, &attr) == 0) {
+#if 0
+				/* Debug output of original and updated memberUid values */
+				char **ary1, **ary2;
+				ary1 = slapi_entry_attr_get_charray(e, "memberUid");
+				ary2 = slapi_entry_attr_get_charray(staged.entries[0], "memberUid");
+
+				slapi_log_error(SLAPI_LOG_FATAL, plugin_id,
+						"original group \"%s\":\n", staged.name);
+				for (int i = 0; ary1 && ary1[i] != NULL; ++i) {
+					slapi_log_error(SLAPI_LOG_FATAL, plugin_id,
+							"\t> %s\n", ary1[i]);
+				}
+				slapi_log_error(SLAPI_LOG_FATAL, plugin_id,
+						"new group \"%s\":\n", staged.name);
+				for (int i = 0; ary2 && ary2[i] != NULL; ++i) {
+					slapi_log_error(SLAPI_LOG_FATAL, plugin_id,
+							"\t> %s\n", ary2[i]);
+				}
+				slapi_ch_array_free(ary2);
+				slapi_ch_array_free(ary1);
+#endif
+
+				(void)slapi_attr_get_valueset(attr, &valueset);
+
+				if (slapi_entry_attr_find(e, IPA_ATTR_MEMBERUID, &attr) == 0) {
+					(void) slapi_entry_attr_delete(e, IPA_ATTR_MEMBERUID);
+				}
+				(void) slapi_entry_add_valueset(e, IPA_ATTR_MEMBERUID, valueset);
+				slapi_valueset_free(valueset);
+			} else {
+				slapi_log_error(SLAPI_LOG_PLUGIN, plugin_id,
+						"group \"%s\" doesn't have memberUid attribute\n", staged.name);
+			}
+			slapi_entry_free(staged.entries[0]);
+	}
+
+	if (staged.entries != NULL) {
+		free(staged.entries);
+	}
+
+	(void)slapi_entry_attr_delete(e, IPA_ATTR_EXTERNAL_MEMBER);
+	free(cbdata.nsswitch_buffer);
+	slapi_ch_free_string(&staged.name);
+}
+#endif
+
 /* Given a map-entry directory entry, determine a key, a value, and extra data
  * to be stored in the map cache, and add them to the map cache. */
 static void
@@ -609,6 +722,9 @@ backend_set_entry_from(Slapi_PBlock *pb, enum backend_entry_source source,
 		slapi_entry_add_string(entry,
 				       "objectClass", "extensibleObject");
 	}
+#ifdef USE_NSSWITCH
+	backend_set_process_external_members(pb, entry, data->common.state, data);
+#endif
 	/* Clean up the entry by doing a round trip through the LDIF parser. */
 	ldif = slapi_entry2str(entry, &len);
 	slapi_entry_free(entry);
@@ -996,7 +1112,7 @@ backend_search_entry_cb(const char *domain, const char *map, bool_t secure,
 			void *backend_data, void *cb_data)
 {
 	Slapi_DN *sdn;
-	Slapi_Entry *entry;
+	Slapi_Entry *entry = NULL; /* prevent to free an uninitialized entry */
 	Slapi_Attr *attr = NULL;
 	struct backend_search_cbdata *cbdata;
 	struct backend_entry_data *entry_data;
@@ -1039,21 +1155,24 @@ backend_search_entry_cb(const char *domain, const char *map, bool_t secure,
 				slapi_sdn_get_ndn(sdn));
 		entry = entry_data->e;
 #ifdef USE_IPA_IDVIEWS
-		entry = slapi_entry_dup(entry_data->e);
 		if (cbdata->idview != NULL) {
-			idview_process_overrides(cbdata, key, map, domain, entry);
-		}
+			entry = slapi_entry_dup(entry_data->e);
+			if (entry != NULL) {
+				idview_process_overrides(cbdata, key, map, domain, entry);
 
-		/* slapi_entry_attr_exists() was introduced only in https://fedorahosted.org/389/ticket/47710 */
-		is_attr_exists = slapi_entry_attr_find(entry, IPA_IDVIEWS_ATTR_ANCHORUUID, &attr) == 0;
+				/* slapi_entry_attr_exists() was introduced only in https://fedorahosted.org/389/ticket/47710 */
+				is_attr_exists = slapi_entry_attr_find(entry, IPA_IDVIEWS_ATTR_ANCHORUUID, &attr) == 0;
 
-		if (is_attr_exists == TRUE) {
-			slapi_entry_attr_delete(entry, IPA_IDVIEWS_ATTR_ANCHORUUID);
-			slapi_entry_delete_string(entry, "objectClass", "ipaOverrideTarget");
+				if (is_attr_exists == TRUE) {
+					slapi_entry_attr_delete(entry, IPA_IDVIEWS_ATTR_ANCHORUUID);
+					slapi_entry_delete_string(entry, "objectClass", "ipaOverrideTarget");
+				}
+			} else {
+				entry = entry_data->e;
+			}
 		}
 #endif
-		slapi_send_ldap_search_entry(cbdata->pb, entry, NULL,
-					     cbdata->attrs, cbdata->attrsonly);
+		backend_entries_to_return_push(cbdata, entry);
 		cbdata->n_entries++;
 
 		if (entry != entry_data->e) {
@@ -1070,7 +1189,7 @@ backend_search_set_cb(const char *group, const char *set, bool_t flag,
 {
 	struct backend_search_cbdata *cbdata;
 	struct backend_set_data *set_data;
-	Slapi_Entry *set_entry;
+	Slapi_Entry *set_entry = NULL ; /* prevent to free an uninitialized entry */
 	int result, n_entries;
 	int n_entries_without_nsswitch;
 	const char *ndn;
@@ -1124,12 +1243,11 @@ backend_search_set_cb(const char *group, const char *set, bool_t flag,
 							 set_data->common.group, set_entry);
 			}
 #endif
-			slapi_send_ldap_search_entry(cbdata->pb, set_entry,
-						     NULL, cbdata->attrs,
-						     cbdata->attrsonly);
+			backend_entries_to_return_push(cbdata, set_entry);
 			cbdata->n_entries++;
 			break;
 		}
+
 		slapi_entry_free(set_entry);
 	}
 
@@ -1201,10 +1319,22 @@ static bool_t
 backend_search_find_set_dn_cb(const char *group, void *cb_data)
 {
 	struct backend_search_cbdata *cbdata;
+	Slapi_DN *group_dn;
 
 	cbdata = cb_data;
+
+	/* Check the group itself. */
+	group_dn = slapi_sdn_new_dn_byval(group);
+	if (slapi_sdn_scope_test(group_dn, cbdata->target_dn,
+				 cbdata->scope) == 1) {
+		cbdata->answer = TRUE;
+		slapi_sdn_free(&group_dn);
+		return TRUE;
+	}
+
 	map_data_foreach_map(cbdata->state, group,
 			     backend_search_find_set_dn_in_group_cb, cb_data);
+	slapi_sdn_free(&group_dn);
 	return TRUE;
 }
 
@@ -1244,7 +1374,7 @@ backend_search_group_cb(const char *group, void *cb_data)
 {
 	struct backend_search_cbdata *cbdata;
 	Slapi_DN *group_dn;
-	Slapi_Entry *group_entry;
+	Slapi_Entry *group_entry = NULL; /* prevent to free an uninitialized entry */
 	int result, n_maps;
 
 	cbdata = cb_data;
@@ -1279,12 +1409,11 @@ backend_search_group_cb(const char *group, void *cb_data)
 				idview_process_overrides(cbdata, NULL, NULL, group, group_entry);
 			}
 #endif
-			slapi_send_ldap_search_entry(cbdata->pb, group_entry,
-						     NULL, cbdata->attrs,
-						     cbdata->attrsonly);
+			backend_entries_to_return_push(cbdata, group_entry);
 			cbdata->n_entries++;
 			break;
 		}
+
 		slapi_entry_free(group_entry);
 	}
 
@@ -1343,23 +1472,168 @@ backend_sch_scope_as_string(int scope)
 	return "";
 }
 
+/* The entries are pushed (added) at the end of the list
+ * so that they will be send in the head->tail order
+ */
+static void
+backend_entries_to_return_push(struct backend_search_cbdata *cbdata, Slapi_Entry *e)
+{
+	struct entries_to_send *e_to_send = NULL;
+	struct cached_entry *entry = NULL;
+	bool_t dont_cache = FALSE;
+	PLHashTable* ht = (PLHashTable*) cbdata->state->cached_entries;
+
+	if ((cbdata == NULL) || (e == NULL)) return;
+
+	e_to_send = (struct entries_to_send *) slapi_ch_calloc(1, sizeof(struct entries_to_send));
+
+	dont_cache = cbdata->state->use_entry_cache ? FALSE : TRUE;
+
+	if (!wrap_rwlock_wrlock(cbdata->state->cached_entries_lock)) {
+		entry = PL_HashTableLookup(ht, slapi_entry_get_ndn(e));
+		if (entry != NULL) {
+			/* There is an entry in the hash table but is it the same? */
+			char *e_modifyTimestamp = slapi_entry_attr_get_charptr(e, "modifyTimestamp");
+			char *entry_modifyTimestamp = slapi_entry_attr_get_charptr(entry->entry, "modifyTimestamp");
+			unsigned long e_usn = slapi_entry_attr_get_ulong(e, "entryUSN");
+			unsigned long entry_usn = slapi_entry_attr_get_ulong(entry->entry, "entryUSN");
+			int res = -1;
+
+			/* Our comparison strategy is following:
+			 * - compare modifyTimestamp values first,
+			 * - if they are the same (modifyTimestamp in slapi-nis is down to a second precision),
+			 *   compare entryUSN values if they exist
+			 * - default to not using the cached entry to be on safe side if both comparisons don't
+			 *   give us a definite answer */
+			if ((e_modifyTimestamp != NULL) && (entry_modifyTimestamp != NULL)) {
+				res = strncmp(e_modifyTimestamp, entry_modifyTimestamp, strlen(e_modifyTimestamp));
+			}
+
+			if ((res == 0) && ((e_usn != 0) && (entry_usn != 0))) {
+				res = e_usn != entry_usn ? 1 : 0;
+			}
+
+			if (res != 0) {
+				/* Cached entry is different, evict it from the hash table */
+				(void) PL_HashTableRemove(ht, slapi_entry_get_ndn(entry->entry));
+
+				/* We don't want to clear the entry because it is still in use by other thread.
+				 * Instead, we'll insert new entry into hash table, let the linked list in other
+				 * search to remove the entry itself, but mark it as non-cached. */
+				entry->not_cached = TRUE;
+				entry = NULL;
+			} else {
+				slapi_log_error(SLAPI_LOG_PLUGIN,
+					cbdata->state->plugin_desc->spd_id,
+					"referenced entry [%s], USNs: %ld vs %ld, [%s] vs [%s]\n",
+					slapi_entry_get_ndn(e), e_usn, entry_usn, e_modifyTimestamp, entry_modifyTimestamp);
+				/* It is the same entry, reference it for us */
+				(void) PR_AtomicIncrement(&entry->refcount);
+			}
+
+			if (e_modifyTimestamp != NULL)
+				slapi_ch_free_string(&e_modifyTimestamp);
+			if (entry_modifyTimestamp != NULL)
+				slapi_ch_free_string(&entry_modifyTimestamp);
+		}
+
+		if (entry == NULL) {
+			/* no cached entry for this DN */
+			entry = (struct cached_entry *) slapi_ch_calloc(1, sizeof(struct cached_entry));
+			entry->entry = slapi_entry_dup(e);
+			entry->not_cached = FALSE;
+			(void) PR_AtomicSet(&entry->refcount, 1);
+			if ((ht != NULL) && (entry->entry != NULL) && (!dont_cache)) {
+				(void) PL_HashTableAdd(ht, slapi_entry_get_ndn(entry->entry), entry);
+			}
+		}
+
+		wrap_rwlock_unlock(cbdata->state->cached_entries_lock);
+	}
+
+	e_to_send->entry = entry;
+	if (cbdata->entries_tail == NULL) {
+		/* First entry in that list */
+		cbdata->entries_tail = e_to_send;
+		cbdata->entries_head = e_to_send;
+	} else {
+		cbdata->entries_tail->next = e_to_send;
+		cbdata->entries_tail = e_to_send;
+	}
+}
+
+static void
+backend_send_mapped_entries(struct backend_search_cbdata *cbdata)
+{
+	struct entries_to_send *e_to_send, *next;
+	PLHashTable* ht = NULL;
+	int i = 0;
+	PRInt32 count;
+
+	if (cbdata == NULL) return;
+	ht = (PLHashTable*) cbdata->state->cached_entries;
+
+	/* iterate from head->tail sending the stored entries */
+	for (e_to_send = cbdata->entries_head, i = 0; e_to_send != NULL; i++) {
+		next = e_to_send->next;
+		if (e_to_send->entry->refcount > 0) {
+			slapi_send_ldap_search_entry(cbdata->pb, e_to_send->entry->entry, NULL,
+						     cbdata->attrs, cbdata->attrsonly);
+
+			/* Clean up entry only if there is no reference to it any more in any outstanding request */
+			wrap_rwlock_wrlock(cbdata->state->cached_entries_lock);
+			count = PR_AtomicDecrement(&e_to_send->entry->refcount);
+			if (count == 0) {
+				if (!e_to_send->entry->not_cached) {
+					(void) PL_HashTableRemove(ht, slapi_entry_get_ndn(e_to_send->entry->entry));
+				}
+				/* free this returned entry */
+				slapi_entry_free(e_to_send->entry->entry);
+				e_to_send->entry->entry = NULL;
+				slapi_ch_free((void **) &e_to_send->entry);
+				e_to_send->entry = NULL;
+			}
+			wrap_rwlock_unlock(cbdata->state->cached_entries_lock);
+		}
+
+		/* Otherwise only free list item */
+		slapi_ch_free((void **) &e_to_send);
+		e_to_send = next;
+	}
+	cbdata->entries_head = NULL;
+	cbdata->entries_tail = NULL;
+}
+
 static int
 backend_search_cb(Slapi_PBlock *pb)
 {
 	struct backend_search_cbdata cbdata;
 	struct backend_staged_search *staged, *next;
-	int i;
+	int i, isroot, ret;
 
 	if (wrap_get_call_level() > 0) {
 		return 0;
 	}
 	memset(&cbdata, 0, sizeof(cbdata));
 	cbdata.pb = pb;
-	slapi_pblock_get(pb, SLAPI_PLUGIN_PRIVATE, &cbdata.state);
-	if (cbdata.state->plugin_base == NULL) {
+	ret = slapi_pblock_get(pb, SLAPI_PLUGIN_PRIVATE, &cbdata.state);
+	if ((ret == -1) || (cbdata.state->plugin_base == NULL)) {
 		/* The plugin was not actually started. */
 		return 0;
 	}
+	if (cbdata.state->ready_to_serve == 0) {
+		/* No data to serve yet */
+		return 0;
+	}
+
+	ret = slapi_pblock_get(pb, SLAPI_REQUESTOR_ISROOT, &isroot);
+
+	if ((ret == -1) || (slapi_op_internal(pb) || (slapi_is_ldapi_conn(pb) && isroot))) {
+		/* The plugin should not engage in internal searches of other
+		 * plugins or ldapi+cn=DM */
+		return 0;
+	}
+
 	slapi_pblock_get(pb, SLAPI_SEARCH_TARGET, &cbdata.target);
 	slapi_pblock_get(pb, SLAPI_SEARCH_SCOPE, &cbdata.scope);
 	slapi_pblock_get(pb, SLAPI_SEARCH_SIZELIMIT, &cbdata.sizelimit);
@@ -1443,6 +1717,8 @@ backend_search_cb(Slapi_PBlock *pb)
 				cbdata.state->plugin_desc->spd_id,
 				"unable to acquire read lock\n");
 	}
+	/* Return existing collected entries */
+	backend_send_mapped_entries(&cbdata);
 	wrap_dec_call_level();
 #ifdef USE_NSSWITCH
 	/* If during search of some sets we staged additional lookups, perform them. */
@@ -1525,6 +1801,8 @@ backend_search_cb(Slapi_PBlock *pb)
 		if (map_rdlock() == 0) {
 			map_data_foreach_domain(cbdata.state, backend_search_group_cb, &cbdata);
 			map_unlock();
+			/* Return newly acquired entries */
+			backend_send_mapped_entries(&cbdata);
 		} else {
 			slapi_log_error(SLAPI_LOG_PLUGIN,
 					cbdata.state->plugin_desc->spd_id,
@@ -1681,10 +1959,10 @@ backend_locate(Slapi_PBlock *pb, struct backend_entry_data **data, const char **
 					if (res == 1) {
 						slapi_rdn_remove_index(rdn, 1);
 						slapi_rdn_add(rdn, "uid", bval.bv_val);
-						slapi_sdn_free(&cbdata.target_dn);
-						cbdata.target_dn = slapi_sdn_set_rdn(scbdata.target_dn, rdn);
-						map_data_foreach_map(cbdata.state, NULL, backend_locate_cb, &cbdata);
 					}
+					slapi_sdn_free(&cbdata.target_dn);
+					cbdata.target_dn = slapi_sdn_set_rdn(scbdata.target_dn, rdn);
+					map_data_foreach_map(cbdata.state, NULL, backend_locate_cb, &cbdata);
 					slapi_ber_bvdone(&bval);
 					slapi_rdn_free(&rdn);
 					idview_free_overrides(&scbdata);
@@ -1753,6 +2031,10 @@ backend_write_cb(Slapi_PBlock *pb, struct plugin_state *state)
 	if (wrap_get_call_level() > 0) {
 		return 0;
 	}
+	if (state->ready_to_serve == 0) {
+		/* No data to serve yet */
+		return 0;
+	}
 
 	wrap_inc_call_level();
 	if (map_rdlock() == 0) {
@@ -1860,6 +2142,10 @@ backend_bind_cb(Slapi_PBlock *pb)
 	}
 
 	slapi_pblock_get(pb, SLAPI_PLUGIN_PRIVATE, &state);
+	if (state->ready_to_serve == 0) {
+		/* No data to serve yet */
+		return 0;
+	}
 	/* The code below handles three separate facts:
 	 * 1. For NSSWITCH-discovered users PAM is responsible for authentication.
 	 *    We want to run PAM auth without any slapi-nis lock taken to avoid
