@@ -1,5 +1,5 @@
 /*
- * Copyright 2008 Red Hat, Inc.
+ * Copyright 2008,2012 Red Hat, Inc.
  *
  * This Program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -293,7 +293,7 @@ bool_t
 map_data_foreach_entry_id(struct plugin_state *state,
 			  const char *domain, const char *map, const char *id,
 			  bool_t (*fn)(const char *domain, const char *map,
-			  	       bool_t secure,
+				       bool_t secure,
 				       const char *key,
 				       unsigned int key_len,
 				       const char *value,
@@ -389,7 +389,10 @@ int
 map_master_name(struct plugin_state *state, const char **master)
 {
 	char *tmp, hostname[HOST_NAME_MAX + 1];
-	if (backend_read_master_name(state, &tmp) == 0) {
+	Slapi_PBlock *pb;
+
+	pb = slapi_pblock_new();
+	if (backend_read_master_name(state, pb, &tmp) == 0) {
 		free(map_data.master);
 		map_data.master = strdup(tmp);
 		backend_free_master_name(state, tmp);
@@ -402,6 +405,7 @@ map_master_name(struct plugin_state *state, const char **master)
 		map_data.master = strdup(hostname);
 	}
 	*master = map_data.master;
+	slapi_pblock_destroy(pb);
 	return 0;
 }
 
@@ -446,11 +450,11 @@ map_match(struct plugin_state *state,
 	if (map == NULL) {
 		return FALSE;
 	}
+	*secure = map->secure;
 	entry = map_data_find_map_entry(state, map, key_len, key, &key_index);
 	if (entry == NULL) {
 		return FALSE;
 	}
-	*secure = map->secure;
 	*value_len = entry->value_len[key_index % entry->n_values];
 	*value = entry->values[key_index % entry->n_values];
 	*id = entry->id;
@@ -483,6 +487,7 @@ map_match_id(struct plugin_state *state,
 	if (map == NULL) {
 		return FALSE;
 	}
+	*secure = map->secure;
 	entry = map_data_find_map_entry_id(state, map, in_id);
 	if (entry == NULL) {
 		return FALSE;
@@ -493,7 +498,6 @@ map_match_id(struct plugin_state *state,
 	if (in_key_index >= entry->n_keys) {
 		return FALSE;
 	}
-	*secure = map->secure;
 	*key_len = entry->key_len[in_key_index];
 	*key = entry->keys[in_key_index];
 	*value_len = entry->value_len[in_key_index % entry->n_values];
@@ -527,11 +531,11 @@ map_first(struct plugin_state *state,
 	if (map == NULL) {
 		return FALSE;
 	}
+	*secure = map->secure;
 	entry = map->entries;
 	if (entry == NULL) {
 		return FALSE;
 	}
-	*secure = map->secure;
 	*first_key_len = entry->key_len[0];
 	*first_key = entry->keys[0];
 	*first_value_len = entry->value_len[0];
@@ -553,7 +557,7 @@ map_next(struct plugin_state *state,
 {
 	struct map *map;
 	struct map_entry *entry;
-	unsigned int key_index;
+	unsigned int key_index, last_instance;
 	*next_key_len = 0;
 	*next_key = NULL;
 	*next_value_len = 0;
@@ -562,11 +566,23 @@ map_next(struct plugin_state *state,
 	if (map == NULL) {
 		return FALSE;
 	}
+	*secure = map->secure;
 	entry = map_data_find_map_entry(state, map, prev_len, prev, &key_index);
 	if (entry == NULL) {
 		return FALSE;
 	}
-	*secure = map->secure;
+	last_instance = entry->n_keys - 1;
+	while (last_instance > key_index) {
+		if ((prev_len != entry->key_len[last_instance]) ||
+		    (memcmp(prev, entry->keys[last_instance], prev_len) != 0)) {
+			last_instance--;
+		} else {
+			break;
+		}
+	}
+	if (last_instance > key_index) {
+		key_index = last_instance;
+	}
 	if (key_index + 1 < entry->n_keys) {
 		*next_key_len = entry->key_len[key_index + 1];
 		*next_key = entry->keys[key_index + 1];
@@ -604,11 +620,11 @@ map_next_id(struct plugin_state *state,
 	if (map == NULL) {
 		return FALSE;
 	}
+	*secure = map->secure;
 	entry = map_data_find_map_entry_id(state, map, prev_id);
 	if (entry == NULL) {
 		return FALSE;
 	}
-	*secure = map->secure;
 	if (prev_key_index + 1 < (int) entry->n_keys) {
 		*next_key_len = entry->key_len[prev_key_index + 1];
 		*next_key = entry->keys[prev_key_index + 1];
@@ -640,7 +656,7 @@ map_data_check_entry(struct plugin_state *state,
 {
 	return (map_data_find_map_entry_id(state,
 					   map_data_find_map(state,
-					   		     domain_name,
+							     domain_name,
 							     map_name),
 					   id) != NULL);
 }
@@ -785,8 +801,8 @@ map_data_unset_map(struct plugin_state *state,
 			/* Close the hole in the array. */
 			domain->n_maps--;
 			if (i != domain->n_maps) {
-				memcpy(&domain->maps[i], &domain->maps[i + 1],
-				       sizeof(*map) * (domain->n_maps - i));
+				memmove(&domain->maps[i], &domain->maps[i + 1],
+				        sizeof(*map) * (domain->n_maps - i));
 			}
 			break;
 		}
@@ -804,10 +820,10 @@ map_data_unset_map(struct plugin_state *state,
 				/* Fill in the hole in the domains array. */
 				map_data.n_domains--;
 				if (i != map_data.n_domains) {
-					memcpy(&map_data.domains[i],
-					       &map_data.domains[i + 1],
-					       sizeof(*domain) *
-					       (map_data.n_domains - i));
+					memmove(&map_data.domains[i],
+					        &map_data.domains[i + 1],
+					        sizeof(*domain) *
+					        (map_data.n_domains - i));
 				}
 				break;
 			}
@@ -862,6 +878,9 @@ map_data_set_map(struct plugin_state *state,
 				/* XXX */
 				return;
 			}
+		} else {
+			/* XXX */
+			return;
 		}
 	}
 	/* Check if the map's already been defined in the domain. */
@@ -903,6 +922,9 @@ map_data_set_map(struct plugin_state *state,
 				/* XXX */
 				return;
 			}
+		} else {
+			/* XXX */
+			return;
 		}
 	} else {
 		/* There's already a map there, we just need to update the
@@ -1196,20 +1218,20 @@ map_data_get_map_size(struct plugin_state *state,
 	return 0;
 }
 
-void
+int
 map_rdlock(void)
 {
-	wrap_rwlock_rdlock(map_data.lock);
+	return wrap_rwlock_rdlock(map_data.lock);
 }
 
-void
+int
 map_wrlock(void)
 {
-	wrap_rwlock_wrlock(map_data.lock);
+	return wrap_rwlock_wrlock(map_data.lock);
 }
 
-void
+int
 map_unlock(void)
 {
-	wrap_rwlock_unlock(map_data.lock);
+	return wrap_rwlock_unlock(map_data.lock);
 }
