@@ -1,5 +1,5 @@
 /*
- * Copyright 2008 Red Hat, Inc.
+ * Copyright 2008,2011,2013 Red Hat, Inc.
  *
  * This Program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,7 +41,7 @@
 #endif
 
 #include <rpc/rpc.h>
-#include <rpcsvc/yp_prot.h>
+#include "../yp/yp.h"
 
 #include "disp-nis.h"
 #include "map.h"
@@ -63,7 +63,13 @@ nis_domain(struct plugin_state *state,
 	char *domain = NULL;
 	*reply_bool = FALSE;
 	if (xdr_string(request_xdrs, &domain, YPMAXDOMAIN)) {
-		map_rdlock();
+		if (map_rdlock() != 0) {
+			slapi_log_error(SLAPI_LOG_FATAL,
+					state->plugin_desc->spd_id,
+					"domain(%s) -> lock error (no reply)\n",
+					domain);
+			goto done_with_lock;
+		}
 		map_supports_domain(state, domain, reply_bool);
 		if (*reply_bool || reply_on_failure) {
 			slapi_log_error(SLAPI_LOG_PLUGIN,
@@ -78,6 +84,8 @@ nis_domain(struct plugin_state *state,
 					domain, *reply_bool ? "TRUE" : "FALSE");
 		}
 		map_unlock();
+done_with_lock:
+		xdr_free((xdrproc_t)xdr_string, (char *) &domain);
 	} else {
 		slapi_log_error(SLAPI_LOG_PLUGIN,
 				state->plugin_desc->spd_id,
@@ -103,24 +111,33 @@ nis_match(struct plugin_state *state,
 	memset(&req_key, 0, sizeof(req_key));
 	memset(reply_val, 0, sizeof(*reply_val));
 	if (xdr_ypreq_key(request_xdrs, &req_key)) {
-		map_rdlock();
+		if (map_rdlock() != 0) {
+			slapi_log_error(SLAPI_LOG_FATAL,
+					state->plugin_desc->spd_id,
+					"match(%s/%s/%.*s) -> "
+					"lock error (no reply)\n",
+					req_key.domain, req_key.map,
+					(int) req_key.key.keydat_len,
+					req_key.key.keydat_val);
+			goto done_with_lock;
+		}
 		if (map_match(state, req_key.domain, req_key.map, &map_secure,
-			      req_key.keydat.keydat_len,
-			      req_key.keydat.keydat_val,
-			      &reply_val->valdat.valdat_len,
-			      (const char **) &reply_val->valdat.valdat_val,
+			      req_key.key.keydat_len,
+			      req_key.key.keydat_val,
+			      &reply_val->val.valdat_len,
+			      (const char **) &reply_val->val.valdat_val,
 			      &entry_id, NULL) &&
 		    (client_secure || !map_secure)) {
 			/* Success! */
-			reply_val->status = YP_TRUE;
+			reply_val->stat = YP_TRUE;
 			slapi_log_error(SLAPI_LOG_PLUGIN,
 					state->plugin_desc->spd_id,
 					"match(%s/%s/%.*s) -> %d\n",
 					req_key.domain,
 					req_key.map,
-					(int) req_key.keydat.keydat_len,
-					req_key.keydat.keydat_val,
-					reply_val->status);
+					(int) req_key.key.keydat_len,
+					req_key.key.keydat_val,
+					reply_val->stat);
 		} else {
 			/* Distinguish between no-such-key and no-such-map for
 			 * reporting the error.  We're making maps which the
@@ -128,10 +145,12 @@ nis_match(struct plugin_state *state,
 			map_supported = FALSE;
 			map_supports_map(state, req_key.domain, req_key.map,
 					 &map_supported, NULL);
-			reply_val->status = map_supported ? YP_NOKEY : YP_NOMAP;
+			reply_val->stat = map_supported ? YP_NOKEY : YP_NOMAP;
 		}
 		(*reply_fn)(state, cdata, reply, reply_xdrs);
 		map_unlock();
+done_with_lock:
+		xdr_free((xdrproc_t)xdr_ypreq_key, (char *) &req_key);
 	} else {
 		/* XXX */
 	}
@@ -155,23 +174,30 @@ nis_first(struct plugin_state *state,
 	memset(&req_nokey, 0, sizeof(req_nokey));
 	memset(reply_key_val, 0, sizeof(*reply_key_val));
 	if (xdr_ypreq_nokey(request_xdrs, &req_nokey)) {
-		map_rdlock();
+		if (map_rdlock() != 0) {
+			slapi_log_error(SLAPI_LOG_FATAL,
+					state->plugin_desc->spd_id,
+					"first(%s/%s) -> "
+					"lock error (no reply)\n",
+					req_nokey.domain, req_nokey.map);
+			goto done_with_lock;
+		}
 		if (map_first(state, req_nokey.domain, req_nokey.map,
 			      &map_secure,
-			      &reply_key_val->keydat.keydat_len,
-			      &reply_key_val->keydat.keydat_val,
-			      &reply_key_val->valdat.valdat_len,
-			      &reply_key_val->valdat.valdat_val,
+			      &reply_key_val->key.keydat_len,
+			      &reply_key_val->key.keydat_val,
+			      &reply_key_val->val.valdat_len,
+			      &reply_key_val->val.valdat_val,
 			      &entry_id, &entry_key_index) &&
 		    (client_secure || !map_secure)) {
 			/* Success! */
-			reply_key_val->status = YP_TRUE;
+			reply_key_val->stat = YP_TRUE;
 			slapi_log_error(SLAPI_LOG_PLUGIN,
 					state->plugin_desc->spd_id,
 					"first(%s/%s) -> %d\n",
 					req_nokey.domain,
 					req_nokey.map,
-					reply_key_val->status);
+					reply_key_val->stat);
 		} else {
 			/* Distinguish between no-such-key and no-such-map for
 			 * reporting the error.  We're making maps which the
@@ -179,11 +205,13 @@ nis_first(struct plugin_state *state,
 			map_supported = FALSE;
 			map_supports_map(state, req_nokey.domain, req_nokey.map,
 					 &map_supported, NULL);
-			reply_key_val->status = map_supported ? YP_NOKEY :
-								YP_NOMAP;
+			reply_key_val->stat = map_supported ? YP_NOKEY :
+							      YP_NOMAP;
 		}
 		(*reply_fn)(state, cdata, reply, reply_xdrs);
 		map_unlock();
+done_with_lock:
+		xdr_free((xdrproc_t)xdr_ypreq_nokey, (char *) &req_nokey);
 	} else {
 		/* XXX */
 	}
@@ -205,25 +233,34 @@ nis_next(struct plugin_state *state,
 	memset(&req_key, 0, sizeof(req_key));
 	memset(reply_key_val, 0, sizeof(*reply_key_val));
 	if (xdr_ypreq_key(request_xdrs, &req_key)) {
-		map_rdlock();
+		if (map_rdlock() != 0) {
+			slapi_log_error(SLAPI_LOG_FATAL,
+					state->plugin_desc->spd_id,
+					"next(%s/%s/%.*s) -> "
+					"lock error (no reply)\n",
+					req_key.domain, req_key.map,
+					req_key.key.keydat_len,
+					req_key.key.keydat_val);
+			goto done_with_lock;
+		}
 		if (map_next(state, req_key.domain, req_key.map, &map_secure,
-			     req_key.keydat.keydat_len,
-			     req_key.keydat.keydat_val,
-			     &reply_key_val->keydat.keydat_len,
-			     &reply_key_val->keydat.keydat_val,
-			     &reply_key_val->valdat.valdat_len,
-			     &reply_key_val->valdat.valdat_val) &&
+			     req_key.key.keydat_len,
+			     req_key.key.keydat_val,
+			     &reply_key_val->key.keydat_len,
+			     &reply_key_val->key.keydat_val,
+			     &reply_key_val->val.valdat_len,
+			     &reply_key_val->val.valdat_val) &&
 		    (client_secure || !map_secure)) {
 			/* Success! */
-			reply_key_val->status = YP_TRUE;
+			reply_key_val->stat = YP_TRUE;
 			slapi_log_error(SLAPI_LOG_PLUGIN,
 					state->plugin_desc->spd_id,
 					"next(%s/%s/%.*s) -> %d\n",
 					req_key.domain,
 					req_key.map,
-					req_key.keydat.keydat_len,
-					req_key.keydat.keydat_val,
-					reply_key_val->status);
+					req_key.key.keydat_len,
+					req_key.key.keydat_val,
+					reply_key_val->stat);
 		} else {
 			/* Distinguish between no-next-key and no-this-key for
 			 * reporting the error so that the client can know when
@@ -231,14 +268,15 @@ nis_next(struct plugin_state *state,
 			 * allowed to read appear to be empty. */
 			if (map_match(state, req_key.domain, req_key.map,
 				      &map_secure,
-				      req_key.keydat.keydat_len,
-				      req_key.keydat.keydat_val,
-				      &reply_key_val->valdat.valdat_len,
-				      (const char **) &reply_key_val->valdat.valdat_val,
+				      req_key.key.keydat_len,
+				      req_key.key.keydat_val,
+				      &reply_key_val->val.valdat_len,
+				      (const char **) &reply_key_val->val.valdat_val,
 				      &entry_id, NULL) &&
 			    (client_secure || !map_secure)) {
 				/* Have data for this key, but not the next. */
-				reply_key_val->status = YP_NOMORE;
+				reply_key_val->stat = YP_NOMORE;
+				map_supported = TRUE;
 			} else {
 				/* No data for this key. Check if we even have
 				 * a map by that name, just to be sure. */
@@ -247,20 +285,22 @@ nis_next(struct plugin_state *state,
 						 req_key.domain,
 						 req_key.map,
 						 &map_supported, NULL);
-				reply_key_val->status = map_supported ?
-							YP_NOKEY : YP_NOMAP;
+				reply_key_val->stat = map_supported ?
+						      YP_NOKEY : YP_NOMAP;
 			}
 			slapi_log_error(SLAPI_LOG_PLUGIN,
 					state->plugin_desc->spd_id,
 					"next(%s/%s/%.*s) -> no %s!\n",
 					req_key.domain,
 					req_key.map,
-					req_key.keydat.keydat_len,
-					req_key.keydat.keydat_val,
+					req_key.key.keydat_len,
+					req_key.key.keydat_val,
 					map_supported ? "more" : "map");
 		}
 		(*reply_fn)(state, cdata, reply, reply_xdrs);
 		map_unlock();
+done_with_lock:
+		xdr_free((xdrproc_t)xdr_ypreq_key, (char *) &req_key);
 	} else {
 		/* XXX */
 	}
@@ -283,27 +323,34 @@ nis_master(struct plugin_state *state,
 	memset(&req_nokey, 0, sizeof(req_nokey));
 	memset(reply_master, 0, sizeof(*reply_master));
 	if (xdr_ypreq_nokey(request_xdrs, &req_nokey)) {
-		map_rdlock();
+		if (map_rdlock() != 0) {
+			slapi_log_error(SLAPI_LOG_FATAL,
+					state->plugin_desc->spd_id,
+					"master(%s/%s) -> "
+					"lock error (no reply)\n",
+					req_nokey.domain, req_nokey.map);
+			goto done_with_lock;
+		}
 		if (map_supports_domain(state, req_nokey.domain, &supported) &&
 		    supported) {
 			if (map_supports_map(state,
 					     req_nokey.domain, req_nokey.map,
 					     &supported, NULL) &&
 			    supported) {
-				reply_master->status = YP_TRUE;
+				reply_master->stat = YP_TRUE;
 				if (map_master_name(state, &master) != 0) {
 					master = "localhost";
 				}
-				reply_master->master = (char *) master;
+				reply_master->peer = (char *) master;
 				slapi_log_error(SLAPI_LOG_PLUGIN,
 						state->plugin_desc->spd_id,
 						"master(%s/%s) -> %s\n",
 						req_nokey.domain,
 						req_nokey.map,
-						reply_master->master);
+						reply_master->peer);
 			} else {
-				reply_master->status = YP_NOMAP;
-				reply_master->master = NULL;
+				reply_master->stat = YP_NOMAP;
+				reply_master->peer = NULL;
 				slapi_log_error(SLAPI_LOG_PLUGIN,
 						state->plugin_desc->spd_id,
 						"master(%s/%s) -> no-map\n",
@@ -311,8 +358,8 @@ nis_master(struct plugin_state *state,
 						req_nokey.map);
 			}
 		} else {
-			reply_master->status = YP_NODOM;
-			reply_master->master = NULL;
+			reply_master->stat = YP_NODOM;
+			reply_master->peer = NULL;
 			slapi_log_error(SLAPI_LOG_PLUGIN,
 					state->plugin_desc->spd_id,
 					"master(%s/%s) -> no-domain\n",
@@ -321,6 +368,8 @@ nis_master(struct plugin_state *state,
 		}
 		(*reply_fn)(state, cdata, reply, reply_xdrs);
 		map_unlock();
+done_with_lock:
+		xdr_free((xdrproc_t)xdr_ypreq_nokey, (char *) &req_nokey);
 	} else {
 		/* XXX */
 	}
@@ -341,12 +390,19 @@ nis_order(struct plugin_state *state,
 	memset(&req_nokey, 0, sizeof(req_nokey));
 	memset(reply_order, 0, sizeof(*reply_order));
 	if (xdr_ypreq_nokey(request_xdrs, &req_nokey)) {
-		map_rdlock();
-		reply_order->status = YP_TRUE;
+		if (map_rdlock() != 0) {
+			slapi_log_error(SLAPI_LOG_FATAL,
+					state->plugin_desc->spd_id,
+					"order(%s/%s) -> "
+					"lock error (no reply)\n",
+					req_nokey.domain, req_nokey.map);
+			goto done_with_lock;
+		}
+		reply_order->stat = YP_TRUE;
 		if (map_order(state, req_nokey.domain, req_nokey.map,
 			      &map_secure, &reply_order->ordernum) &&
 			      (client_secure || !map_secure)) {
-			reply_order->status = YP_TRUE;
+			reply_order->stat = YP_TRUE;
 			slapi_log_error(SLAPI_LOG_PLUGIN,
 					state->plugin_desc->spd_id,
 					"order(%s/%s) -> %d\n",
@@ -354,7 +410,7 @@ nis_order(struct plugin_state *state,
 					req_nokey.map,
 					reply_order->ordernum);
 		} else {
-			reply_order->status = YP_FALSE;
+			reply_order->stat = YP_FALSE;
 			slapi_log_error(SLAPI_LOG_PLUGIN,
 					state->plugin_desc->spd_id,
 					"order(%s/%s) -> ?\n",
@@ -363,6 +419,8 @@ nis_order(struct plugin_state *state,
 		}
 		(*reply_fn)(state, cdata, reply, reply_xdrs);
 		map_unlock();
+done_with_lock:
+		xdr_free((xdrproc_t)xdr_ypreq_nokey, (char *) &req_nokey);
 	} else {
 		/* XXX */
 	}
@@ -422,19 +480,26 @@ nis_maplist(struct plugin_state *state,
 	struct nis_maplist_cbdata cbdata;
 	memset(reply_maplist, 0, sizeof(*reply_maplist));
 	if (xdr_string(request_xdrs, &domain, YPMAXDOMAIN)) {
-		map_rdlock();
+		if (map_rdlock() != 0) {
+			slapi_log_error(SLAPI_LOG_FATAL,
+					state->plugin_desc->spd_id,
+					"maplist(%s) -> "
+					"lock error (no reply)\n",
+					domain);
+			goto done_with_lock;
+		}
 		cbdata.list = NULL;
 		cbdata.client_secure = client_secure;
 		map_data_foreach_map(state, domain, nis_maplist_cb, &cbdata);
-		reply_maplist->status = YP_TRUE;
-		reply_maplist->list = cbdata.list;
-		if (reply_maplist->list == NULL) {
+		reply_maplist->stat = YP_TRUE;
+		reply_maplist->maps = cbdata.list;
+		if (reply_maplist->maps == NULL) {
 			slapi_log_error(SLAPI_LOG_PLUGIN,
 					state->plugin_desc->spd_id,
 					"maplist(%s) -> (none)\n",
 					domain);
 		} else {
-			for (list = reply_maplist->list;
+			for (list = reply_maplist->maps;
 			     list != NULL;
 			     list = list->next) {
 				slapi_log_error(SLAPI_LOG_PLUGIN,
@@ -446,6 +511,8 @@ nis_maplist(struct plugin_state *state,
 		(*reply_fn)(state, cdata, reply, reply_xdrs);
 		nis_free_maplist_cb_result(&cbdata);
 		map_unlock();
+done_with_lock:
+		xdr_free((xdrproc_t)xdr_string, (char *) &domain);
 	} else {
 		/* XXX */
 	}
@@ -522,8 +589,8 @@ nis_all(struct plugin_state *state,
 	struct ypresp_all *reply_all, void **continuation_cookie)
 {
 	struct ypreq_nokey req_nokey;
-	keydat_t *reply_key;
-	valdat_t *reply_val;
+	keydat *reply_key;
+	valdat *reply_val;
 	struct nis_all_cookie *cookie;
 	enum nis_all_cookie_state next_state;
 	bool_t map_supported, map_secure, stop;
@@ -531,8 +598,8 @@ nis_all(struct plugin_state *state,
 	int entry_key_index;
 
 	memset(&req_nokey, 0, sizeof(req_nokey));
-	reply_key = &reply_all->ypresp_all_u.val.keydat;
-	reply_val = &reply_all->ypresp_all_u.val.valdat;
+	reply_key = &reply_all->ypresp_all_u.val.key;
+	reply_val = &reply_all->ypresp_all_u.val.val;
 	if (xdr_ypreq_nokey(request_xdrs, &req_nokey)) {
 		/* Take ownership of the cookie data. */
 		if (continuation_cookie) {
@@ -547,14 +614,21 @@ nis_all(struct plugin_state *state,
 			cookie = nis_all_make_cookie(cookie_bad, NULL, 0);
 		}
 		/* Check if we even support the map. */
-		map_rdlock();
+		if (map_rdlock() != 0) {
+			slapi_log_error(SLAPI_LOG_FATAL,
+					state->plugin_desc->spd_id,
+					"all(%s/%s) -> "
+					"lock error (no reply)\n",
+					req_nokey.domain, req_nokey.map);
+			goto done_with_lock;
+		}
 		map_supported = FALSE;
 		if (!map_supports_map(state, req_nokey.domain, req_nokey.map,
 				      &map_supported, NULL) ||
 		    !map_supported) {
 			/* No entries? No-such-map final status. */
 			reply_all->more = TRUE;
-			reply_all->ypresp_all_u.val.status = YP_NOMAP;
+			reply_all->ypresp_all_u.val.stat = YP_NOMAP;
 			reply_key->keydat_len = 0;
 			reply_val->valdat_len = 0;
 			/* Encode the reply header so that we can queue the
@@ -562,14 +636,24 @@ nis_all(struct plugin_state *state,
 	                xdr_replymsg(reply_xdrs, reply);
 			/* End of data. */
 			reply_all->more = FALSE;
-			xdr_ypresp_all(reply_xdrs, reply_all);
-			/* Queue the entire response. */
-			if (!(*reply_fragment_fn)(state, cdata,
-						  reply, reply_xdrs,
-						  FALSE, TRUE)) {
+			if (xdr_ypresp_all(reply_xdrs, reply_all)) {
+				/* Queue the entire response. */
+				if (!(*reply_fragment_fn)(state, cdata,
+							  reply, reply_xdrs,
+							  FALSE, TRUE)) {
+					slapi_log_error(SLAPI_LOG_PLUGIN,
+							state->plugin_desc->spd_id,
+							"all(%s/%s) - "
+							"error queueing "
+							"error response\n",
+							req_nokey.domain,
+							req_nokey.map);
+				}
+			} else {
 				slapi_log_error(SLAPI_LOG_PLUGIN,
 						state->plugin_desc->spd_id,
-						"all(%s/%s) - error queueing "
+						"all(%s/%s) - "
+						"error building "
 						"error response\n",
 						req_nokey.domain,
 						req_nokey.map);
@@ -581,7 +665,7 @@ nis_all(struct plugin_state *state,
 		for (stop = FALSE; stop == FALSE;) {
 			bool_t found, skip;
 			xdr_setpos(reply_xdrs, 0);
-			memset(reply_all, 0, sizeof(reply_all));
+			memset(reply_all, 0, sizeof(*reply_all));
 			/* Follow any instructions we left for this iteration.
 			 */
 			switch (cookie->state) {
@@ -613,7 +697,7 @@ nis_all(struct plugin_state *state,
 							reply_key->keydat_val);
 					skip = FALSE;
 					reply_all->more = TRUE;
-					reply_all->ypresp_all_u.val.status = YP_TRUE;
+					reply_all->ypresp_all_u.val.stat = YP_TRUE;
 					next_state = cookie_next;
 				} else {
 					/* Don't reply, just move to end-of-map
@@ -676,7 +760,7 @@ nis_all(struct plugin_state *state,
 					 * one. */
 					skip = FALSE;
 					reply_all->more = TRUE;
-					reply_all->ypresp_all_u.val.status = YP_TRUE;
+					reply_all->ypresp_all_u.val.stat = YP_TRUE;
 					next_state = cookie_next;
 				} else {
 					/* Don't reply, just move to end-of-map
@@ -743,7 +827,7 @@ nis_all(struct plugin_state *state,
 							cookie->key_index);
 					skip = FALSE;
 					reply_all->more = TRUE;
-					reply_all->ypresp_all_u.val.status = YP_TRUE;
+					reply_all->ypresp_all_u.val.stat = YP_TRUE;
 					next_state = cookie_next;
 				} else {
 					/* Don't reply, just move to end-of-map
@@ -783,7 +867,7 @@ nis_all(struct plugin_state *state,
 				memset(reply_key, 0, sizeof(*reply_key));
 				memset(reply_val, 0, sizeof(*reply_key));
 				reply_all->more = TRUE;
-				reply_all->ypresp_all_u.val.status = YP_NOMORE;
+				reply_all->ypresp_all_u.val.stat = YP_NOMORE;
 				slapi_log_error(SLAPI_LOG_PLUGIN,
 						state->plugin_desc->spd_id,
 						"all(%s/%s) no entries\n",
@@ -809,7 +893,7 @@ nis_all(struct plugin_state *state,
 				memset(reply_key, 0, sizeof(*reply_key));
 				memset(reply_val, 0, sizeof(*reply_key));
 				reply_all->more = TRUE;
-				reply_all->ypresp_all_u.val.status = YP_NOMORE;
+				reply_all->ypresp_all_u.val.stat = YP_NOMORE;
 				slapi_log_error(SLAPI_LOG_PLUGIN,
 						state->plugin_desc->spd_id,
 						"all(%s/%s) end-of-map\n",
@@ -855,6 +939,8 @@ nis_all(struct plugin_state *state,
 			}
 		}
 		map_unlock();
+done_with_lock:
+		xdr_free((xdrproc_t)xdr_ypreq_nokey, (char *) &req_nokey);
 		/* Return the cookie if we can, else destroy it. */
 		if (continuation_cookie) {
 			*continuation_cookie = cookie;
@@ -881,7 +967,7 @@ nis_process_request(struct plugin_state *state,
 	AUTH *request_auth, *reply_auth;
 	char auth_buf[MAX_AUTH_BYTES];
 	struct rpc_msg request, reply;
-	int superuser, auth_flavor, auth_len;
+	int auth_flavor, auth_len;
 	struct ypresp_val reply_val;
 	struct ypresp_key_val reply_key_val;
 	struct ypresp_all reply_all;
@@ -895,6 +981,8 @@ nis_process_request(struct plugin_state *state,
 	memset(&reply_xdrs, 0, sizeof(reply_xdrs));
 	memset(&request, 0, sizeof(request));
 	memset(&reply, 0, sizeof(reply));
+	memset(&auth_buf, 0, sizeof(auth_buf));
+	memset(reply_buf, 0, reply_buf_size);
 
 	/* Parse the client request and make sure it looks like an RPC. */
 	xdrmem_create(&request_xdrs, request_buf, request_buflen, XDR_DECODE);
@@ -937,26 +1025,22 @@ nis_process_request(struct plugin_state *state,
 			slapi_log_error(SLAPI_LOG_PLUGIN,
 					state->plugin_desc->spd_id,
 					"validated auth_sys creds\n");
-			superuser = 0;
 			break;
 		case AUTH_NONE:
 			slapi_log_error(SLAPI_LOG_PLUGIN,
 					state->plugin_desc->spd_id,
 					"validated \"none\" creds\n");
-			superuser = 0;
 			break;
 		default:
 			slapi_log_error(SLAPI_LOG_PLUGIN,
 					state->plugin_desc->spd_id,
 					"validated other creds\n");
-			superuser = 0;
 			break;
 		}
 	} else {
 		slapi_log_error(SLAPI_LOG_PLUGIN,
 				state->plugin_desc->spd_id,
 				"failed to validate client creds\n");
-		superuser = 0;
 	}
 	auth_destroy(request_auth);
 
@@ -1144,6 +1228,7 @@ nis_process_request(struct plugin_state *state,
 			    &reply, &reply_xdrs, &reply_maplist);
 		goto sent_reply;
 		break;
+#ifdef YPPROC_NEWXFR
 	case YPPROC_NEWXFR:
 		slapi_log_error(SLAPI_LOG_PLUGIN,
 				state->plugin_desc->spd_id,
@@ -1151,6 +1236,7 @@ nis_process_request(struct plugin_state *state,
 		reply.rm_reply.rp_stat = MSG_DENIED;
 		reply.rm_reply.rp_rjct.rj_stat = RPC_MISMATCH;
 		break;
+#endif
 	}
 
 send_reply:
@@ -1160,6 +1246,7 @@ sent_reply:
 	xdr_destroy(&reply_xdrs);
 
 done:
+	xdr_free((xdrproc_t)xdr_callmsg, (char *) &request);
 	xdr_destroy(&request_xdrs);
 	return;
 }
