@@ -44,6 +44,7 @@
 
 #ifdef HAVE_DIRSRV_SLAPI_PLUGIN_H
 #include <nspr.h>
+#include <plhash.h>
 #include <nss.h>
 #include <dirsrv/slapi-plugin.h>
 #else
@@ -100,6 +101,7 @@ plugin_startup(Slapi_PBlock *pb)
 {
 	/* Populate the maps and data. */
 	struct plugin_state *state;
+	Slapi_Entry *plugin_entry = NULL;
 	slapi_pblock_get(pb, SLAPI_PLUGIN_PRIVATE, &state);
 	slapi_pblock_get(pb, SLAPI_TARGET_DN, &state->plugin_base);
 	slapi_log_error(SLAPI_LOG_PLUGIN, state->plugin_desc->spd_id,
@@ -107,16 +109,39 @@ plugin_startup(Slapi_PBlock *pb)
 			state->plugin_base ? "\"" : "",
 			state->plugin_base ? state->plugin_base : "NULL",
 			state->plugin_base ? "\"" : "");
-	/* Populate the tree of fake entries. */
-	backend_startup(pb, state);
 	state->pam_lock = wrap_new_rwlock();
 	backend_nss_init_context((struct nss_ops_ctx**) &state->nss_context);
+	if ((slapi_pblock_get(pb, SLAPI_PLUGIN_CONFIG_ENTRY, &plugin_entry) == 0) &&
+	    (plugin_entry != NULL)) {
+		state->use_entry_cache = backend_shr_get_vattr_boolean(state, plugin_entry,
+									"slapi-entry-cache",
+									1);
+	}
+	state->cached_entries_lock = wrap_new_rwlock();
+	wrap_rwlock_wrlock(state->cached_entries_lock);
+	state->cached_entries = PL_NewHashTable(0, PL_HashString, PL_CompareStrings, PL_CompareValues, 0, 0);
+	wrap_rwlock_unlock(state->cached_entries_lock);
+	/* Populate the tree of fake entries. */
+	backend_startup(pb, state);
 	/* Note that the plugin is ready to go. */
 	slapi_log_error(SLAPI_LOG_PLUGIN, plugin_description.spd_id,
 			"plugin startup completed\n");
 	return 0;
 }
 
+static PRIntn
+remove_cached_entries_cb(PLHashEntry *he, PRIntn i, void *arg)
+{
+	struct cached_entry *e = (struct cached_entry*) he->value;
+	if (e != NULL) {
+		if (e->entry != NULL) {
+			slapi_entry_free(e->entry);
+		}
+		slapi_ch_free((void **) &e);
+	}
+	return HT_ENUMERATE_REMOVE;
+}
+
 static int
 plugin_shutdown(Slapi_PBlock *pb)
 {
@@ -126,6 +151,15 @@ plugin_shutdown(Slapi_PBlock *pb)
 	wrap_free_rwlock(state->pam_lock);
 	state->pam_lock = NULL;
 	backend_nss_free_context((struct nss_ops_ctx**) &state->nss_context);
+	if (state->cached_entries != NULL) {
+		wrap_rwlock_wrlock(state->cached_entries_lock);
+		PL_HashTableEnumerateEntries(state->cached_entries, remove_cached_entries_cb, NULL);
+		PL_HashTableDestroy(state->cached_entries);
+		state->cached_entries = NULL;
+		wrap_rwlock_unlock(state->cached_entries_lock);
+		wrap_free_rwlock(state->cached_entries_lock);
+		state->cached_entries_lock = NULL;
+	}
 	state->plugin_base = NULL;
 	slapi_log_error(SLAPI_LOG_PLUGIN, state->plugin_desc->spd_id,
 			"plugin shutdown completed\n");
