@@ -33,6 +33,7 @@ struct plugin_state {
 	Slapi_ComponentId *plugin_identity;
 	Slapi_PluginDesc *plugin_desc;
 	unsigned int use_be_txns: 1;
+	PRInt32 ready_to_serve;
 
 	/* NIS-specific data. */
 	struct wrapped_thread *tid;
@@ -47,6 +48,9 @@ struct plugin_state {
 	/* Schema compat-specific data. */
 	struct wrapped_rwlock *pam_lock;
 	void *nss_context;
+	int use_entry_cache;
+	void *cached_entries;
+	struct wrapped_rwlock *cached_entries_lock;
 };
 
 #endif
